@@ -37,8 +37,22 @@
 /*
  * This is needed to expose pthread_mutexattr_settype and PTHREAD_MUTEX_DEFAULT
  * on older pthreads implementations
+ *
+ * This is excluded on SunOS due to it making the code compile in < XPG6 mode
+ * which is not supported with a C99 compiler.
  */
+#ifndef __sun
 #define _XOPEN_SOURCE 700
+#endif
+
+/*
+ * This is needed to expose issetugid, getresuid, and getresgid, which are
+ * hidden with the _XOPEN_SOURCE setting above
+ */
+#ifdef __FreeBSD__
+#undef __BSD_VISIBLE
+#define __BSD_VISIBLE 1
+#endif
 
 #include "compat.h"
 #include "debug.h"
@@ -49,6 +63,9 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef OS_UNIX
+#include <unistd.h>
+#endif
 
 /*-
  * Portions of this file are covered by the following copyright:
@@ -90,8 +107,6 @@
 
 #ifdef OS_UNIX
 
-#include <unistd.h>
-
 #if defined (HAVE_PROGRAM_INVOCATION_SHORT_NAME) && !HAVE_DECL_PROGRAM_INVOCATION_SHORT_NAME
 extern char *program_invocation_short_name;
 #endif
@@ -100,6 +115,19 @@ extern char *program_invocation_short_name;
 extern char *__progname;
 #endif
 
+#ifdef __linux__
+/* This symbol is also defined in library.c so as to be freed by the library
+ * destructor.  If weak symbols are not supported nor library.c is not linked we
+ * simply leak the memory allocated with realpath().  */
+#ifdef __GNUC__
+extern char *p11_program_realpath;
+
+char *p11_program_realpath __attribute__((weak));
+#else
+static char *p11_program_realpath;
+#endif
+#endif
+
 const char *
 getprogname (void)
 {
@@ -124,14 +152,14 @@ getprogname (void)
 		 * Logic borrowed from:
 		 * <https://github.com/mesa3d/mesa/commit/759b94038987bb983398cd4b1d2cb1c8f79817a9>.
 		 */
-		static char *buf;
+		if (!p11_program_realpath)
+			p11_program_realpath = realpath ("/proc/self/exe", NULL);
 
-		if (!buf)
-			buf = realpath ("/proc/self/exe", NULL);
-
-		if (buf && strncmp (buf, name, strlen (buf)) == 0)
+		if (p11_program_realpath &&
+		    strncmp (p11_program_realpath, name,
+			     strlen (p11_program_realpath)) == 0)
 			/* Use the executable path if the prefix matches. */
-			name = strrchr (buf, '/') + 1;
+			name = strrchr (p11_program_realpath, '/') + 1;
 		else
 			/* Otherwise fall back to
 			 * program_invocation_short_name. */
@@ -939,7 +967,6 @@ fdwalk (int (* cb) (void *data, int fd),
         void *data)
 {
 	int open_max;
-	long num;
 	int res = 0;
 	int fd;
 
@@ -948,13 +975,16 @@ fdwalk (int (* cb) (void *data, int fd),
 #endif
 
 #ifdef __linux__
-	struct dirent *de;
-	char *end;
 	DIR *dir;
 
 	dir = opendir ("/proc/self/fd");
 	if (dir != NULL) {
+		struct dirent *de;
+
 		while ((de = readdir (dir)) != NULL) {
+			char *end;
+			long num;
+
 			end = NULL;
 			num = (int) strtol (de->d_name, &end, 10);
 
