@@ -39,6 +39,9 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_UN_H
+#  include <sys/un.h>
+#endif
 
 #if !defined(__cplusplus) && (__GNUC__ > 2)
 #define GNUC_PRINTF(x, y) __attribute__((__format__(__printf__, x, y)))
@@ -185,7 +188,6 @@ void        p11_mmap_close  (p11_mmap *map);
 #include <pthread.h>
 #include <dlfcn.h>
 #include <time.h>
-#include <unistd.h>
 
 typedef pthread_mutex_t p11_mutex_t;
 
@@ -382,6 +384,13 @@ int        fdwalk           (int (* cb) (void *data, int fd),
 #undef HAVE_STRERROR_L
 #endif
 
+#ifndef SUN_LEN
+
+#define SUN_LEN(sa) (sizeof(struct sockaddr_un) \
+	- sizeof((sa)->sun_path) + strlen((sa)->sun_path))
+
+#endif
+
 int        p11_ascii_tolower (int c);
 int        p11_ascii_toupper (int c);
 
