@@ -66,6 +66,7 @@ typedef struct {
 	p11_rpc_client_vtable *vtable;
 	unsigned int initialized_forkid;
 	bool initialize_done;
+	uint8_t version;
 } rpc_client;
 
 /* Allocator for call session buffers */
@@ -755,10 +756,33 @@ rpc_C_Initialize (CK_X_FUNCTION_LIST *self,
 	assert (module->vtable->connect != NULL);
 	ret = (module->vtable->connect) (module->vtable, reserved);
 
+	if (ret == CKR_OK) {
+		module->version = P11_RPC_PROTOCOL_VERSION_MAXIMUM;
+		ret = (module->vtable->authenticate) (module->vtable,
+						      &module->version);
+
+#if P11_RPC_PROTOCOL_VERSION_MAXIMUM > 0
+		/* If the server is too old to support version negotiation
+		 * (i.e., not accepting version bytes other than 0), try to
+		 * reconnect and reauthenticate with version 0 */
+		if (ret != CKR_OK) {
+			assert (module->vtable->disconnect != NULL);
+			(module->vtable->disconnect) (module->vtable, reserved);
+			ret = (module->vtable->connect) (module->vtable, reserved);
+
+			module->version = 0;
+			ret = (module->vtable->authenticate) (module->vtable,
+							      &module->version);
+		}
+#endif
+	}
+
 	/* Successfully initialized */
 	if (ret == CKR_OK) {
 		module->initialized_forkid = p11_forkid;
 		module->initialize_done = true;
+		p11_debug ("authenticated with protocol version %u",
+			   module->version);
 
 	/* Server doesn't exist, initialize but don't call */
 	} else if (ret == CKR_DEVICE_REMOVED) {
