@@ -51,6 +51,13 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(x) dgettext(PACKAGE_NAME, x)
+#else
+#define _(x) (x)
+#endif
+
 static bool
 load_attached_extension (p11_dict *attached,
                          p11_dict *asn1_defs,
@@ -67,7 +74,7 @@ load_attached_extension (p11_dict *attached,
 
 	ext = p11_asn1_decode (asn1_defs, "PKIX1.Extension", der, len, message);
 	if (ext == NULL) {
-		p11_message ("couldn't parse attached certificate extension: %s", message);
+		p11_message (_("couldn't parse attached certificate extension: %s"), message);
 		return false;
 	}
 
@@ -138,7 +145,7 @@ load_attached_extensions (p11_enumerate *ex,
 	}
 
 	if (rv != CKR_OK && rv != CKR_CANCEL) {
-		p11_message ("couldn't load attached extensions for certificate: %s", p11_kit_strerror (rv));
+		p11_message (_("couldn't load attached extensions for certificate: %s"), p11_kit_strerror (rv));
 		p11_dict_free (attached);
 		attached = NULL;
 	}
@@ -185,16 +192,16 @@ check_trust_flags (p11_enumerate *ex)
 	int flags = 0;
 
 	/* If no extract trust flags, then just continue */
-	if (!(ex->flags & (P11_ENUMERATE_ANCHORS | P11_ENUMERATE_BLACKLIST)))
+	if (!(ex->flags & (P11_ENUMERATE_ANCHORS | P11_ENUMERATE_BLOCKLIST)))
 		return true;
 
-	/* Is this a blacklisted directly? */
+	/* Is this a distrusted directly? */
 	if (p11_attrs_find_bool (ex->attrs, CKA_X_DISTRUSTED, &distrusted) && distrusted)
-		flags = P11_ENUMERATE_BLACKLIST;
+		flags = P11_ENUMERATE_BLOCKLIST;
 
-	/* Is it blacklisted elsewhere? then prevent it from being an anchor */
-	else if (p11_dict_get (ex->blacklist_public_key, ex->attrs) ||
-	         p11_dict_get (ex->blacklist_issuer_serial, ex->attrs))
+	/* Is it distrusted elsewhere? then prevent it from being an anchor */
+	else if (p11_dict_get (ex->blocklist_public_key, ex->attrs) ||
+	         p11_dict_get (ex->blocklist_issuer_serial, ex->attrs))
 		flags = 0;
 
 	/* Otherwise it might be an anchor? */
@@ -262,7 +269,7 @@ extract_certificate (p11_enumerate *ex)
 	                                ex->cert_der, ex->cert_len, message);
 
 	if (!ex->cert_asn) {
-		p11_message ("couldn't parse certificate: %s", message);
+		p11_message (_("couldn't parse certificate: %s"), message);
 		return false;
 	}
 
@@ -299,7 +306,7 @@ extract_info (p11_enumerate *ex)
 
 	/* The attributes couldn't be loaded */
 	if (rv != CKR_OK && rv != CKR_ATTRIBUTE_TYPE_INVALID && rv != CKR_ATTRIBUTE_SENSITIVE) {
-		p11_message ("couldn't load attributes: %s", p11_kit_strerror (rv));
+		p11_message (_("couldn't load attributes: %s"), p11_kit_strerror (rv));
 		return false;
 	}
 
@@ -310,7 +317,7 @@ extract_info (p11_enumerate *ex)
 	/* If a certificate then  */
 	if (ex->flags & P11_ENUMERATE_CORRELATE) {
 		if (ex->klass != CKO_CERTIFICATE) {
-			p11_message ("skipping non-certificate object");
+			p11_message (_("skipping non-certificate object"));
 			return false;
 		}
 
@@ -390,7 +397,7 @@ on_iterate_load_filter (p11_kit_iter *iter,
 }
 
 /*
- * Various skip lookup tables, used for blacklists and collapsing
+ * Various skip lookup tables, used for blocklists and collapsing
  * duplicate entries.
  *
  * The dict hash/lookup callbacks are special cased
@@ -432,7 +439,7 @@ issuer_serial_hash (const void *data)
 }
 
 static bool
-blacklist_load (p11_enumerate *ex)
+blocklist_load (p11_enumerate *ex)
 {
 	p11_kit_iter *iter;
 	CK_BBOOL distrusted = CK_TRUE;
@@ -463,28 +470,28 @@ blacklist_load (p11_enumerate *ex)
 
 		/*
 		 * Fail "safe" in that first failure doesn't cause ignoring
-		 * the remainder of the blacklist.
+		 * the remainder of the blocklist.
 		 */
 		rv = p11_kit_iter_load_attributes (iter, attrs, 3);
 		if (rv != CKR_OK) {
-			p11_message ("couldn't load blacklist: %s", p11_kit_strerror (rv));
+			p11_message (_("couldn't load blocklist: %s"), p11_kit_strerror (rv));
 			continue;
 		}
 
-		/* A blacklisted item with an issuer and serial number */
+		/* A distrusted item with an issuer and serial number */
 		issuer = p11_attrs_find_valid (attrs, CKA_ISSUER);
 		serial = p11_attrs_find_valid (attrs, CKA_SERIAL_NUMBER);
 		if (issuer != NULL && serial != NULL) {
 			key = p11_attrs_build (NULL, issuer, serial, NULL);
-			if (!key || !p11_dict_set (ex->blacklist_issuer_serial, key, "x"))
+			if (!key || !p11_dict_set (ex->blocklist_issuer_serial, key, "x"))
 				return_val_if_reached (false);
 		}
 
-		/* A blacklisted item with a public key */
+		/* A distrusted item with a public key */
 		public_key = p11_attrs_find_valid (attrs, CKA_PUBLIC_KEY_INFO);
 		if (public_key != NULL) {
 			key = p11_attrs_build (NULL, public_key, NULL);
-			if (!public_key || !p11_dict_set (ex->blacklist_public_key, key, "x"))
+			if (!public_key || !p11_dict_set (ex->blocklist_public_key, key, "x"))
 				return_val_if_reached (false);
 		}
 	}
@@ -495,7 +502,7 @@ blacklist_load (p11_enumerate *ex)
 	if (rv == CKR_CANCEL)
 		return true;
 
-	p11_message ("couldn't load blacklist: %s", p11_kit_strerror (rv));
+	p11_message (_("couldn't load blocklist: %s"), p11_kit_strerror (rv));
 	return false;
 }
 
@@ -509,13 +516,13 @@ p11_enumerate_init (p11_enumerate *ex)
 	ex->iter = p11_kit_iter_new (NULL, 0);
 	return_if_fail (ex->iter != NULL);
 
-	ex->blacklist_public_key = p11_dict_new (public_key_hash, public_key_equal,
+	ex->blocklist_public_key = p11_dict_new (public_key_hash, public_key_equal,
 	                                         p11_attrs_free, NULL);
-	return_if_fail (ex->blacklist_public_key);
+	return_if_fail (ex->blocklist_public_key);
 
-	ex->blacklist_issuer_serial = p11_dict_new (issuer_serial_hash, issuer_serial_equal,
+	ex->blocklist_issuer_serial = p11_dict_new (issuer_serial_hash, issuer_serial_equal,
 	                                            p11_attrs_free, NULL);
-	return_if_fail (ex->blacklist_issuer_serial);
+	return_if_fail (ex->blocklist_issuer_serial);
 
 	p11_kit_iter_add_callback (ex->iter, on_iterate_load_filter, ex, NULL);
 }
@@ -530,10 +537,10 @@ p11_enumerate_cleanup (p11_enumerate *ex)
 
 	p11_dict_free (ex->already_seen);
 	ex->already_seen = NULL;
-	p11_dict_free (ex->blacklist_public_key);
-	ex->blacklist_public_key = NULL;
-	p11_dict_free (ex->blacklist_issuer_serial);
-	ex->blacklist_issuer_serial = NULL;
+	p11_dict_free (ex->blocklist_public_key);
+	ex->blocklist_public_key = NULL;
+	p11_dict_free (ex->blocklist_issuer_serial);
+	ex->blocklist_issuer_serial = NULL;
 
 	p11_dict_free (ex->asn1_defs);
 	ex->asn1_defs = NULL;
@@ -569,19 +576,19 @@ p11_enumerate_opt_filter (p11_enumerate *ex,
 
 	if (strncmp (option, "pkcs11:", 7) == 0) {
 		if (ex->uri != NULL) {
-			p11_message ("a PKCS#11 URI has already been specified");
+			p11_message (_("a PKCS#11 URI has already been specified"));
 			return false;
 		}
 
 		ex->uri = p11_kit_uri_new ();
 		ret = p11_kit_uri_parse (option, P11_KIT_URI_FOR_OBJECT_ON_TOKEN_AND_MODULE, ex->uri);
 		if (ret != P11_KIT_URI_OK) {
-			p11_message ("couldn't parse pkcs11 uri filter: %s", option);
+			p11_message (_("couldn't parse pkcs11 uri filter: %s"), option);
 			return false;
 		}
 
 		if (p11_kit_uri_any_unrecognized (ex->uri))
-			p11_message ("uri contained unrecognized components, nothing will be extracted");
+			p11_message (_("uri contained unrecognized components, nothing will be extracted"));
 
 		p11_kit_iter_set_uri (ex->iter, ex->uri);
 		ex->num_filters++;
@@ -594,11 +601,11 @@ p11_enumerate_opt_filter (p11_enumerate *ex,
 
 	} else if (strcmp (option, "trust-policy") == 0) {
 		attrs = p11_attrs_build (NULL, &certificate, &x509, NULL);
-		ex->flags |= P11_ENUMERATE_ANCHORS | P11_ENUMERATE_BLACKLIST | P11_ENUMERATE_COLLAPSE;
+		ex->flags |= P11_ENUMERATE_ANCHORS | P11_ENUMERATE_BLOCKLIST | P11_ENUMERATE_COLLAPSE;
 
-	} else if (strcmp (option, "blacklist") == 0) {
+	} else if (strcmp (option, "blocklist") == 0) {
 		attrs = p11_attrs_build (NULL, &certificate, &x509, NULL);
-		ex->flags |= P11_ENUMERATE_BLACKLIST | P11_ENUMERATE_COLLAPSE;
+		ex->flags |= P11_ENUMERATE_BLOCKLIST | P11_ENUMERATE_COLLAPSE;
 
 	} else if (strcmp (option, "certificates") == 0) {
 		attrs = p11_attrs_build (NULL, &certificate, &x509, NULL);
@@ -608,7 +615,7 @@ p11_enumerate_opt_filter (p11_enumerate *ex,
 		attrs = p11_attrs_build (NULL, NULL);
 
 	} else {
-		p11_message ("unsupported or unrecognized filter: %s", option);
+		p11_message (_("unsupported or unrecognized filter: %s"), option);
 		return false;
 	}
 
@@ -657,7 +664,7 @@ p11_enumerate_opt_purpose (p11_enumerate *ex,
 	} else if (is_valid_oid_rough (option)) {
 		oid = option;
 	} else {
-		p11_message ("unsupported or unregonized purpose: %s", option);
+		p11_message (_("unsupported or unrecognized purpose: %s"), option);
 		return false;
 	}
 
@@ -699,15 +706,15 @@ p11_enumerate_ready (p11_enumerate *ex,
 	if (!ex->modules)
 		return false;
 	if (ex->modules[0] == NULL)
-		p11_message ("no modules containing trust policy are registered");
+		p11_message (_("no modules containing trust policy are registered"));
 
 	/*
-	 * If loading anchors, then the caller expects that the blacklist is
-	 * "applied" and any anchors on the blacklist are taken out. This is
-	 * for compatibility with software that does not support blacklists.
+	 * If loading anchors, then the caller expects that the blocklist is
+	 * "applied" and any anchors on the blocklist are taken out. This is
+	 * for compatibility with software that does not support blocklists.
 	 */
 	if (ex->flags & P11_ENUMERATE_ANCHORS) {
-		if (!blacklist_load (ex))
+		if (!blocklist_load (ex))
 			return false;
 	}
 
